@@ -64,10 +64,10 @@ static void gdm_server_child_handler (gint);
 static char * get_font_path (const char *display);
 
 /* Global vars */
-static GdmDisplay *d = NULL;
-static gboolean server_signal_notified = FALSE;
 static int server_signal_pipe[2];
-static int gdm_in_signal = 0;
+static GdmDisplay *d                   = NULL;
+static gboolean server_signal_notified = FALSE;
+static int gdm_in_signal               = 0;
 
 static void do_server_wait (GdmDisplay *d);
 static gboolean setup_server_wait (GdmDisplay *d);
@@ -951,6 +951,26 @@ rotate_logs (const char *dname)
 	g_free (fname);
 }
 
+static void
+gdm_server_add_xserver_args (GdmDisplay *d, char **argv)
+{
+	int count;
+	char **args;
+	int len;
+	int i;
+
+	len = gdm_vector_len (argv);
+	g_shell_parse_argv (d->xserver_session_args, &count, &args, NULL);
+	argv = g_renew (char *, argv, len + count + 1);
+
+	for (i=0; i < count;i++) {
+		argv[len++] = g_strdup(args[i]);
+	}
+
+	argv[len] = NULL;
+	g_strfreev (args);
+}
+
 GdmXserver *
 gdm_server_resolve (GdmDisplay *disp)
 {
@@ -1177,6 +1197,12 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 				     &argc,
 				     &argv);
 
+    /* Do not support additional session arguments with Xnest. */
+    if (d->type != TYPE_FLEXI_XNEST) {
+	    if (d->xserver_session_args)
+		    gdm_server_add_xserver_args (d, argv);
+    }
+
     command = g_strjoinv (" ", argv);
 
     /* Fork into two processes. Parent remains the gdm process. Child
@@ -1456,85 +1482,6 @@ gdm_server_child_handler (int signal)
 }
 
 
-/**
- * gdm_server_alloc:
- * @id: Local display number
- * @command: Command line for starting the X server
- *
- * Allocate display structure for a local X server
- */
-
-GdmDisplay *
-gdm_server_alloc (gint id, const gchar *command)
-{
-    gchar hostname[1024];
-    GdmDisplay *d;
-
-    hostname[1023] = '\0';
-    if (gethostname (hostname, 1023) == -1)
-	    strcpy (hostname, "localhost.localdomain");
-
-    d = g_new0 (GdmDisplay, 1);
-
-    d->logout_action = GDM_LOGOUT_ACTION_NONE;
-
-    d->authfile = NULL;
-    d->authfile_gdm = NULL;
-    d->auths = NULL;
-    d->userauth = NULL;
-    d->command = g_strdup (command);
-    d->cookie = NULL;
-    d->dispstat = DISPLAY_UNBORN;
-    d->greetpid = 0;
-    d->name = g_strdup_printf (":%d", id);  
-    d->hostname = g_strdup (hostname);
-    /* Not really used for not XDMCP */
-    memset (&(d->addr), 0, sizeof (d->addr));
-    d->dispnum = id;
-    d->servpid = 0;
-    d->servstat = SERVER_DEAD;
-    d->sesspid = 0;
-    d->slavepid = 0;
-    d->type = TYPE_STATIC;
-    d->attached = TRUE;
-    d->sessionid = 0;
-    d->acctime = 0;
-    d->dsp = NULL;
-    d->screenx = 0; /* xinerama offset */
-    d->screeny = 0;
-
-    d->handled = TRUE;
-    d->tcp_disallowed = FALSE;
-
-    d->priority = 0;
-    d->vt = -1;
-
-    d->x_servers_order = -1;
-
-    d->last_loop_start_time = 0;
-    d->last_start_time = 0;
-    d->retry_count = 0;
-    d->sleep_before_run = 0;
-    d->login = NULL;
-    d->preset_user = NULL;
-
-    d->timed_login_ok = FALSE;
-
-    d->slave_notify_fd = -1;
-    d->master_notify_fd = -1;
-
-    d->xsession_errors_bytes = 0;
-    d->xsession_errors_fd = -1;
-    d->session_output_fd = -1;
-
-    d->chooser_output_fd = -1;
-    d->chooser_last_line = NULL;
-
-    d->theme_name = NULL;
-
-    return d;
-}
-
 void
 gdm_server_whack_clients (Display *dsp)
 {
