@@ -18,6 +18,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -33,9 +35,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <gnome.h>
-
-#include <config.h>
+#include <glib/gi18n.h>
 
 #ifdef HAVE_XINPUT
 #include <X11/extensions/XInput.h>
@@ -45,7 +45,7 @@
  * Note that CONFIGFILE will have to be moved to somewhere more generic
  * if this module is ever moved outside of gdm.
  */
-#define CONFIGFILE EXPANDED_SYSCONFDIR "/gdm/modules/AccessKeyMouseEvents"
+#define CONFIGFILE GDMCONFDIR "/modules/AccessKeyMouseEvents"
 #define	iseol(ch)	((ch) == '\r' || (ch) == '\f' || (ch) == '\0' || \
 			(ch) == '\n')
 
@@ -96,7 +96,8 @@ typedef struct {
 
 static int xinput_types[N_INPUT_TYPES];
 
-extern char     **environ;
+extern char **environ;
+
 static gboolean debug_gestures = FALSE;
 static GSList   *gesture_list  = NULL;
 static int      lineno         = 0;
@@ -168,7 +169,8 @@ init_xinput (GdkDisplay *display, GdkWindow *root)
 	    syslog (LOG_WARNING, "checking %d input devices...", num_devices);
 
 	for (i=0; i < num_devices; i++) {
-		if (devices[i].use == IsXExtensionDevice)
+		if (devices[i].use == IsXExtensionDevice &&
+		    devices[i].num_classes > 0)
 		{
 			device = XOpenDevice (GDK_DISPLAY_XDISPLAY (display),
 				devices[i].id);
@@ -206,6 +208,7 @@ init_xinput (GdkDisplay *display, GdkWindow *root)
 			}
 		}
 	}
+	XFreeDeviceList (devices);
 
 	if (debug_gestures)
 	    syslog (LOG_WARNING, "%d event types available", number);
@@ -484,7 +487,7 @@ parse_line (gchar *buf)
 	}
 
 	/*
-	 * Find servcice. Permit blank space so arguments can be supplied.
+	 * Find service. Permit blank space so arguments can be supplied.
 	 */
 	keyservice = strtok (NULL, "\n\r\f");
 	if (keyservice == NULL) {
@@ -897,14 +900,33 @@ gestures_filter (GdkXEvent *gdk_xevent,
 			seq_count = 0;
 			for (act_li = curr_gesture->actions;
 			     act_li != NULL; act_li = act_li->next) {
-
 				gchar *action = (gchar *)act_li->data;
+#ifdef HAVE_CTRUN
+				gchar *ctrun;
+#endif
+
 				g_return_val_if_fail (action != NULL, GDK_FILTER_CONTINUE);
+
+#ifdef HAVE_CTRUN
+				ctrun = g_strdup_printf (
+					"/bin/sh -c \"/usr/bin/ctrun -l child -i none %s\"",
+					action);
+				if (!g_shell_parse_argv (ctrun, NULL, &argv, NULL)) {
+					g_free (ctrun);
+					continue;
+				}
+
+				g_free (ctrun);
+#else
 				if (!g_shell_parse_argv (action, NULL, &argv, NULL))
 					continue;
+#endif
 
 				envp = get_exec_environment (xevent);
 
+				if (debug_gestures)
+				    syslog (LOG_WARNING, "Action is %s", action);
+
 				retval = g_spawn_async (NULL,
 							argv,
 							envp,
